import re
import cv2 as cv

# The commented code bellow would have been used, but due to unresolved issues with CNN it will not be used.
# from keras.models import load_model
from cv_chess_functions import (read_img,
                               canny_edge,
                               hough_line,
                               h_v_lines,
                               line_intersections,
                               cluster_points,
                               augment_points,
                               write_crop_images,
                               grab_cell_files,
                               atoi)
import os

os.makedirs('data','raw_data', exist_ok=True)
# This function resizes the frame by scaling it by dimensions.
# This is a pre-made function made by project_MYM.

# The percentage value determines the new image size based on the input image. 
# Changing this percentage value may be needed if working with several different dimension of images.
def rescale_frame(frame, percent=200): 
    width = int(frame.shape[1] * (percent / 100))
    height = int(frame.shape[0] * (percent / 100))
    dim =  (width, height)
    return cv.resize(frame, dim, interpolation=cv.INTER_AREA)

# Find the number(s) in a text.
# This is a pre-made function made by project_MYM
def natural_keys(text):
    return [atoi(c) for c in re.split('(\d+)', text)]

"""
This function creates a binary string of 1 and 0 from an image
It takes in a list of images, list of cropped tiles
and finds the largest contour in it to see if it contains a piece
"""
def detect_piece_in_square(img_list):
    binary_string=''
    for j in range(len(img_list)):
        cropped_src = cv.imread(img_list[j])

        y = 10
        x = 10
        w = 50
        h = 50
        roi = cropped_src[y:y+h, x:x+w] #crop the edges away as they can contain parts of neighbouring tiles
        # Convert image to gray and blur it
        cropped_src_gray = cv.cvtColor(roi, cv.COLOR_BGR2GRAY)
        cropped_src_gray = cv.blur(cropped_src_gray, (3,3))
        SE = cv.getStructuringElement(cv.MORPH_CROSS,(5,5))
        closed = cv.morphologyEx(cropped_src_gray, cv.MORPH_CLOSE,SE) #Use closeing to remove some smaller contours and make sure the large contours is not split
        if j > 31:
            thresh, bw = cv.threshold(closed,50,255,cv.THRESH_BINARY)
        if j < 32:
            thresh, bw = cv.threshold(closed,160,255,cv.THRESH_BINARY)
        contours, _ = cv.findContours(bw, cv.RETR_TREE, cv.CHAIN_APPROX_SIMPLE)
    
        p=0
        for cnt in contours:
            if cnt.size > 35: #Checks if the contour is larg enough to be a piece
              p=1
              
        if p == 1:
            binary_string = binary_string + '1'
            #Piece detected
        else:
            binary_string = binary_string + '0'
            #No piece detected   
    return binary_string

# Loads in the CNN model
# The commented line of code bellow would load the CNN model from a file. Due to unresolved issues, it will not be used.
#model = load_model('model_Res_net.h5')

# Code that selects the web-camera for recording.
cap = cv.VideoCapture(1)

# Code that capture frame-by-frame images from the recording.
ret, frame = cap.read()

# This function detects and crops the chess board as well as each individual tile it has. 
def detect_tiles(img):
    src = cv.imread(img)
    src= rescale_frame(src) # Calls for the rescale function to adjust the size of the input image.
    
    # Converts image to grayscale and blurs it.
    src_gray = cv.cvtColor(src, cv.COLOR_BGR2GRAY)
    src_gray = cv.blur(src_gray, (3,3))
    
    threshold = 100 #Variable that determines the thresholds of the edge detector.
    canny_output = cv.Canny(src_gray, threshold, threshold)
    #Detects contours and saves them.
    contours, _ = cv.findContours(canny_output, cv.RETR_TREE, cv.CHAIN_APPROX_SIMPLE)
    
    # Creates an array to store contours in 
    contour_list = []
        
    for cnt in contours:
        # Gets the contour size
        area = cv.contourArea(cnt)
        # Add a tuple of the contour and its size to the array.
        contour_list.append((cnt, area))
    
    # Sorts the list based on size.
    sorted_list = sorted(contour_list, key=lambda x: x[1])
    # Creates a window for the largest contour
    for i in range(-1,0):
        x,y,w,h=cv.boundingRect(sorted_list[i][0])
        roi=src[y:y+h, x:x+w] #Saving the contour as a region of interest. 
        
        # Saves the frame for further work.
        cv.imwrite('frame.jpg', roi)  
        
        # Low-level CV techniques (grayscale & blur)
        # read_img() is a pre-made function made by project_MYM.
        img, gray_blur = read_img('frame.jpg')  
        
        # Applies edge detection to the blurred image.
        # canny_edge() is a pre-made function made by project_MYM.
        edges = canny_edge(gray_blur)
        
        # Detects lines based on the edge detection.
        # hough_lines() is a pre-made function made by project_MYM.
        lines = hough_line(edges)
        
        # Separates the lines into vertical lines and horizontal lines.
        # h_v_lines() is a pre-made function made by project_MYM.
        h_lines, v_lines = h_v_lines(lines)
        
        # Finds and clusters the intersecting points based on horizontal and vertical lines
        # line_intersections() and cluster_points() are pre-made functions made by project_MYM.
        intersection_points = line_intersections(h_lines, v_lines)
        points = cluster_points(intersection_points)
        
        # Determines the final coordinates of points on the board.
        # augment_points() is a pre-made function made by project_MYM.
        points = augment_points(points)
        
        # Crops the tiles of the board a organizes them into a sorted list.
        # write_crop_images() is a pre-made function made by project_MYM.
        x_list = write_crop_images(img, points, 0)

        # Gets the name of all saved tiles and sorts them by utilizing the function natural_keys.
        # The tiles are then processed in the function detect_piece_in_square() defined earlier in the code.
        # grab_cell_files() is a pre-made function made by project_MYM.
        img_filename_list = grab_cell_files()
        img_filename_list.sort(key=natural_keys)
        binary_string = detect_piece_in_square(img_filename_list)  
        return binary_string
        
    cv.waitKey() 
    cv.destroyAllWindows()   
