# -*- coding: utf-8 -*-
"""
Created on Fri Apr 16 13:20:15 2021

@author: jansk
"""
import keras
from keras import layers
from keras import models
from keras.optimizers import Adam
from keras.applications.inception_resnet_v2 import InceptionResNetV2
from keras.preprocessing.image import ImageDataGenerator

datagen = ImageDataGenerator(
        rotation_range=5,
        # width_shift_range=0.1,
        # height_shift_range=0.1,
        rescale=1./255,
        horizontal_flip=True,
        fill_mode='nearest')

test_datagen = ImageDataGenerator(rescale=1./255)

data_dir = r'C:\Users\jansk\Dataingenienor\BachelorOppgave\KerasData'

train_gen = datagen.flow_from_directory(
    data_dir + '/train',
    target_size = (224, 224),
    batch_size = 32,
    class_mode = 'categorical',
    color_mode = 'rgb',
    shuffle=True  
)

test_gen = test_datagen.flow_from_directory(
    data_dir +'/validation',
    target_size = (224, 224),
    batch_size = 32,
    class_mode = 'categorical',
    color_mode = 'rgb',
    shuffle=False 
)

base_model = InceptionResNetV2(include_top=False, weights='imagenet', input_shape=(224, 224, 3))
x = base_model.output
x = layers.GlobalAveragePooling2D()(x)
x = layers.Dense(1024, activation='relu')(x)
predictions = layers.Dense(13, activation='softmax')(x) # New softmax layer
model = models.Model(inputs=base_model.input, outputs=predictions)

# we chose to train the top 2 inception blocks
# we will freeze the first 249 layers and unfreeze the rest
for layer in model.layers[:249]:
   layer.trainable = False
for layer in model.layers[249:]:
   layer.trainable = True

adam = Adam(lr=0.0001)  
model.compile(adam, loss='categorical_crossentropy', metrics=['accuracy'])

epochs = 10

history = model.fit(
    train_gen, 
    epochs=epochs,
    steps_per_epoch = 60,
    batch_size=32,
    verbose = 1,
    validation_steps=20,
    validation_data=test_gen)

model.save_weights('model_Res_net_weights_224.h5')
model.save('model_Res_net_224.h5')